"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Paging = Paging;
const react_1 = __importDefault(require("react"));
const components_1 = require("@storybook/components");
const theming_1 = require("@storybook/theming");
const types_js_1 = require("../../../../types.js");
const StyledButton = (0, theming_1.styled)(components_1.Button)({
    transform: 'none',
    marginLeft: '8px',
});
function Paging(props) {
    const renderItem = (item, index) => {
        switch (item) {
            case '.': {
                return (react_1.default.createElement(StyledButton, { disabled: true, key: `dots${index < 5 ? 'Left' : 'Right'}` }, '...'));
            }
            case 'forward': {
                const disabled = props.activePage === props.pagesCount;
                return (react_1.default.createElement(StyledButton, { outline: true, disabled: disabled, onClick: disabled
                        ? types_js_1.noop
                        : () => {
                            goToPage(props.activePage + 1);
                        }, key: "forward" },
                    react_1.default.createElement("span", null,
                        "Next ",
                        react_1.default.createElement(components_1.Icons, { icon: "arrowright" }))));
            }
            default: {
                return (react_1.default.createElement(StyledButton, { outline: true, secondary: props.activePage === item, key: item, onClick: () => {
                        goToPage(item);
                    } }, item));
            }
        }
    };
    const goToPage = (pageNumber) => {
        if (1 <= pageNumber && pageNumber !== props.activePage && pageNumber <= props.pagesCount) {
            props.onPageChange(pageNumber);
        }
    };
    return react_1.default.createElement("div", null, getItems(props.activePage, props.pagesCount).map(renderItem));
}
function getItems(active, total) {
    const result = [];
    const left = Math.max(Math.min(active - 2, total - 4), 1);
    const right = Math.min(Math.max(5, active + 2), total);
    const hasLeftDots = left > 3;
    const from = hasLeftDots ? left : 1;
    const hasRightDots = right < total - 2;
    const to = hasRightDots ? right : total;
    if (hasLeftDots) {
        result.push(1, '.');
    }
    for (let i = from; i <= to; ++i) {
        result.push(i);
    }
    if (hasRightDots) {
        result.push('.');
    }
    if (hasRightDots && isFinite(total)) {
        result.push(total);
    }
    return result.concat('forward');
}
//# sourceMappingURL=Paging.js.map